<?php

App::uses('EvNewsletterSignupsAppModel', 'EvNewsletterSignups.Model');

class Signup extends EvNewsletterSignupsAppModel {

	public $displayField = 'email';

/**
 * validation rules
 */
	public $validate = array(
		'email' => array(
			'required' => array(
				'rule' => 'notBlank',
				'message' => 'Email cannot be left blank'
			),
			'email' => array(
				'rule' => 'email',
				'message' => 'You must enter a valid email address'
			),
			'unique' => array(
				'rule' => 'isUnique',
				'message' => 'A user with that email address already exists'
			)
		)
	);

/**
 * afterFind callback
 * decode the extras if set
 *
 */
	public function afterFind($results, $primary = false) {
		$results = parent::afterFind($results, $primary);

		foreach ($results as $key => $item) {
			if (! empty($item['Signup']['extras'])) {
				$results[$key]['Signup']['extras'] = json_decode($item['Signup']['extras'], true);
			}
		}

		return $results;
	}

/**
 * afterSave callback
 * (optional) send subscriber details to Mailchimp
 * (optional) send subscriber details to CampaignMonitor
 */
	public function afterSave($created, $options = array()) {
		$mailchimpConfig = Configure::read('EvNewsletterSignups.mailchimp');
		$campaignMonitorConfig = Configure::read('EvNewsletterSignups.campaignMonitor');

		// Subscribe to Mailchimp list
		if (is_array($mailchimpConfig)) {
			$this->_subscribeWithMailchimp($this->data, $mailchimpConfig);
		}

		// Subscribe to Campaign Monitor list
		if (is_array($campaignMonitorConfig)) {
			$this->_subscribeWithCampaignMonitor($this->data, $campaignMonitorConfig);
		}
	}

/**
 * Adds a subscriber to a Mailchimp list
 *
 * @param array data - The POST-ed mail data
 * @param array config - The Mailchimp config
 * @return null
 */
	protected function _subscribeWithMailchimp($data, $config) {
		// Set config options and init MailChimp
		Configure::write('Mailchimp', $config['pluginConfig']);
		$Mailchimp = ClassRegistry::init('Mailchimp.Mailchimp');
		$MailchimpSubscriber = ClassRegistry::init('Mailchimp.MailchimpSubscriber');

		// Prep Subscriber Data array
		$subscriber = array(
			'email' => $data['Signup']['email']
		);

		// Prep Mailchimp Options array
		$options = array(
			'doubleOptin' => false,
			'updateExisting' => false
		);

		// Parse any extras and add to Subscriber Data array
		if (!empty($data['Signup']['extras'])) {
			$extras = json_decode($data['Signup']['extras'], true);

			$subscriber = array_merge($subscriber, $extras);
		}

		// Call Mailchimp > Subscribe function
		$MailchimpSubscriber->subscribe($subscriber, $options);
	}

/**
 * Adds a subscriber to a Campaign Monitor list
 *
 * @param array data - The POST-ed mail data
 * @param array config - The Campaign Monitor config
 * @return null
 */
	protected function _subscribeWithCampaignMonitor($data, $config) {
		$campaignMonitorDir = ROOT . DS . 'Vendor' . DS . 'campaignmonitor' . DS . 'createsend-php' . DS;
		require_once $campaignMonitorDir . 'csrest_general.php';

		$auth = array('api_key' => $config['pluginConfig']['apiKey']);
		$wrap = new CS_REST_Subscribers($config['pluginConfig']['defaultListId'], $auth);

		// Prep Subscriber Data array
		$subscriber = array(
			'EmailAddress' => $data['Signup']['email'],
			'Resubscribe' => true
		);

		// Parse any extras
		$extras = array();
		if (!empty($data['Signup']['extras'])) {
			$extras = json_decode($data['Signup']['extras'], true);

			foreach ($extras as $key => $value) {
				$subscriber['CustomFields'][] = array(
					'Key' => $key,
					'Value' => $value
				);
			}
		}

		// Include name?
		if (isset($extras[$config['defaultFields']['name']])) {
			$subscriber['Name'] = $extras[$config['defaultFields']['name']];
		}

		$result = $wrap->add($subscriber);
	}
}
